@@ -2347,8 +2347,8 @@ gst_deinterlace_getcaps (GstDeinterlace * self, GstPad * pad, GstCaps * filter)
   gst_caps_unref (tmp);
   tmp = NULL;
 
-  for (len = gst_caps_get_size (caps); len > 0; len--) {
-    GstStructure *s = gst_caps_get_structure (caps, len - 1);
+  for (len = gst_caps_get_size (tmp2); len > 0; len--) {
+    GstStructure *s = gst_caps_get_structure (tmp2, len - 1);
 
     if (pad == self->sinkpad)
       gst_structure_remove_field (s, "interlace-mode");
@@ -2508,9 +2508,19 @@ gst_deinterlace_setcaps (GstDeinterlace * self, GstPad * pad, GstCaps * caps)
   GstCaps *srccaps = NULL;
   GstVideoInterlaceMode interlacing_mode;
   gint fps_n, fps_d;
+  GstCaps *current_caps;
 
   gst_pad_check_reconfigure (self->srcpad);
 
+  if ((current_caps = gst_pad_get_current_caps (pad))) {
+    if (gst_caps_is_equal (caps, current_caps)) {
+      GST_DEBUG_OBJECT (pad, "Got same caps again, returning");
+      gst_caps_unref (current_caps);
+      return TRUE;
+    }
+    gst_deinterlace_reset_history (self, FALSE);
+  }
+
   if (self->locking != GST_DEINTERLACE_LOCKING_NONE) {
     if (self->low_latency == -1)
       self->low_latency = gst_deinterlace_get_latency (self);
