@@ -19,18 +19,18 @@
 
 /**
  * SECTION:element-rtpL24pay
- * @title: rtpL24pay
  * @see_also: rtpL24depay
  *
  * Payload raw 24-bit audio into RTP packets according to RFC 3190, section 4.
  * For detailed information see: http://www.rfc-editor.org/rfc/rfc3190.txt
  *
- * ## Example pipeline
+ * <refsect2>
+ * <title>Example pipeline</title>
  * |[
  * gst-launch-1.0 -v audiotestsrc ! audioconvert ! rtpL24pay ! udpsink
  * ]| This example pipeline will payload raw audio. Refer to
  * the rtpL24depay example to depayload and play the RTP stream.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,7 +42,6 @@
 #include <gst/audio/audio.h>
 #include <gst/rtp/gstrtpbuffer.h>
 
-#include "gstrtpelements.h"
 #include "gstrtpL24pay.h"
 #include "gstrtpchannels.h"
 
@@ -80,8 +79,6 @@ gst_rtp_L24_pay_handle_buffer (GstRTPBasePayload * basepayload,
 
 #define gst_rtp_L24_pay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpL24Pay, gst_rtp_L24_pay, GST_TYPE_RTP_BASE_AUDIO_PAYLOAD);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpL24pay, "rtpL24pay",
-    GST_RANK_SECONDARY, GST_TYPE_RTP_L24_PAY, rtp_element_init (plugin));
 
 static void
 gst_rtp_L24_pay_class_init (GstRtpL24PayClass * klass)
@@ -96,10 +93,10 @@ gst_rtp_L24_pay_class_init (GstRtpL24PayClass * klass)
   gstrtpbasepayload_class->get_caps = gst_rtp_L24_pay_getcaps;
   gstrtpbasepayload_class->handle_buffer = gst_rtp_L24_pay_handle_buffer;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_L24_pay_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_L24_pay_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_L24_pay_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_L24_pay_sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP audio payloader", "Codec/Payloader/Network/RTP",
@@ -238,3 +235,10 @@ gst_rtp_L24_pay_handle_buffer (GstRTPBasePayload * basepayload,
   return GST_RTP_BASE_PAYLOAD_CLASS (parent_class)->handle_buffer (basepayload,
       buffer);
 }
+
+gboolean
+gst_rtp_L24_pay_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "rtpL24pay",
+      GST_RANK_SECONDARY, GST_TYPE_RTP_L24_PAY);
+}
