@@ -144,6 +144,9 @@ struct _GstRTSPStream {
   /* session */
   GObject      *session;
 
+  /* srtp key management */
+  GstMIKEYMessage *mikey;
+
   /* bandwidth */
   guint         as_bandwidth;
   guint         rs_bandwidth;
@@ -238,6 +241,7 @@ struct _GstRTSPSrc {
   gboolean          do_retransmission;
   gint              ntp_time_source;
   gchar            *user_agent;
+  GstClockTime      max_rtcp_rtp_time_diff;
 
   /* state */
   GstRTSPState       state;
